#ifndef SO_QT_FREE_VIEWER_H
#define SO_QT_FREE_VIEWER_H

///////////////////////////////////////////////////////////////////////////////
//  SoTerrain
///////////////////////////////////////////////////////////////////////////////
/// Prohle pro voln pohyb ve scn pomoc myi a klvesnice.
/// \file SoQtFreeViewer.h
/// \author Radek Barto - xbarto33
/// \date 06.03.2006
///
/// Prohle pro voln pohyb ve scn pomoc myi a klvesnice.
/// Zobraz okno se scnou, ve kter je mono se pohybovat pomoc myi a
/// klvesnice. Kurzor myi je oknem zachycen a skryt, take je mono otet
/// pohledem do vech smr dokola. Pozici kamery lze ovldat tmito klvesami:
/// <DL>
///   <DT>W</DT><DD>posun kamery kupedu</DD>
///   <DT>S</DT><DD>posun kamery dozadu</DD>
///   <DT>A</DT><DD>posun kamery doleva</DD>
///   <DT>D</DT><DD>posun kamery doprava</DD>
///   <DT>Q</DT><DD>posun kamery nahoru</DD>
///   <DT>E</DT><DD>posun kamery dol</DD>
///   <DT>LEFT</DT><DD>natoen kamery doleva</DD>
///   <DT>RIGHT</DT><DD>natoen kamery doprava</DD>
///   <DT>UP</DT><DD>natoen kamery nahoru</DD>
///   <DT>DOWN</DT><DD>natoen kamery dol</DD>
///   <DT>PGUP</DT><DD>naklonn kamery doleva</DD>
///   <DT>PGDOWN</DT><DD>naklonn kamery doprava</DD>
/// </DL>
///////////////////////////////////////////////////////////////////////////////
//
// THIS SOFTWARE IS NOT COPYRIGHTED
//
// This source code is offered for use in the public domain.
//  You may use, modify or distribute it freely.
//
// This source code is distributed in the hope that it will be useful but
// WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
// DISCLAIMED.  This includes but is not limited to warranties of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// If you find the source code useful, authors will kindly welcome
// if you give them credit and keep their names with their source code.
//
///////////////////////////////////////////////////////////////////////////////

#include <Inventor/SbBasic.h>
#include <Inventor/SbLinear.h>
#include <Inventor/nodes/SoCamera.h>
#include <Inventor/events/SoEvent.h>
#include <Inventor/events/SoLocation2Event.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/events/SoMouseButtonEvent.h>

#include <Inventor/Qt/SoQt.h>
#include <Inventor/Qt/SoQtCursor.h>
#include <Inventor/Qt/viewers/SoQtViewer.h>

#include <qcursor.h>
#include <qwidget.h>
#include <qpoint.h>
#include <qsize.h>

#include <debug.h>

/** Prohle pro voln pohyb ve scn pomoc myi a klvesnice.
Zobraz okno se scnou, ve kter je mono se pohybovat pomoc myi a
klvesnice. Kurzor myi je oknem zachycen a skryt, take je mono otet
pohledem do vech smr dokola. Pozici kamery lze ovldat tmito klvesami:
<DL>
  <DT>W</DT><DD>posun kamery kupedu</DD>
  <DT>S</DT><DD>posun kamery dozadu</DD>
  <DT>A</DT><DD>posun kamery doleva</DD>
  <DT>D</DT><DD>posun kamery doprava</DD>
  <DT>Q</DT><DD>posun kamery nahoru</DD>
  <DT>E</DT><DD>posun kamery dol</DD>
  <DT>LEFT</DT><DD>natoen kamery doleva</DD>
  <DT>RIGHT</DT><DD>natoen kamery doprava</DD>
  <DT>UP</DT><DD>natoen kamery nahoru</DD>
  <DT>DOWN</DT><DD>natoen kamery dol</DD>
  <DT>PGUP</DT><DD>naklonn kamery doleva</DD>
  <DT>PGDOWN</DT><DD>naklonn kamery doprava</DD>
</DL>*/
class SoQtFreeViewer : public SoQtViewer
{
  SOQT_OBJECT_HEADER(SoQtFreeViewer, SoQtViewer);
  public:
    /** Konstruktor.
    Vytvo instanci tdy ::SoQtFreeViewer jako potomka okna \p parent a
    nastav jeho vnitn jmno na hodnotu \p name. V ppad, e nen specifikovn
    parametr \p name, je jmno ureno automaticky. Parametr \p emdbed slou
    k uren, zda m bt prohle vloen do svho rodie, i zda m vytvoit svj
    vlastn shell.
    \param parent Ukazatel na rodiovsk okno.
    \param name Jmno prohlee.
    \param embed Pznak vloen prohlee do svho rodie. */
    SoQtFreeViewer(QWidget * parent = NULL, const char * name = NULL,
      SbBool embed = TRUE);
    /** Destruktor.
    Zru instanci tdy ::SoQtFreeViewer. */
    ~SoQtFreeViewer();
    /** Vrt citlivost myi.
    Vrt nastavenou hodnotu citlivosti myi prohlee.
    \return Citlivost myi prohlee. */
    float getMouseSensitivity() const;
    /** Nastav citlivost myi.
    Nastav citlivost myi prohlee na hodnotu \p sensitivity.
    \param sensitivity Nov hodnota citlivosti myi prohlee. */
    void setMouseSensitivity(const float sensitivity);
    /** Vrt pznak invertovan myi.
    Vrt nastavenou hodnotu pznaku invertovan myi prohlee.
    \return Pznak invertovan myi. */
    SbBool getInvertMouse() const;
    /** Nastav pznak invertovan myi.
    Nastav pznak invertovan myi prohlee na hodnotu \p enabled.
    \param enabled Nov hodnota pznaku invertovan myi. */
    void setInvertMouse(const SbBool enabled);
  protected:
    /* Metody. */
    /** Konstruktor.
    Vytvo instanci tdy ::SoQtFreeViewer jako potomka okna \p parent a
    nastav jeho vnitn jmno na hodnotu \p name. V ppad, e nen specifikovn
    parametr \p name, je jmno ureno automaticky. Parametr \p emdbed slou
    k uren, zda m bt prohle vloen do svho rodie, i zda m vytvoit svj
    vlastn shell. Parametr \p build se pouv pro zajitn sprvnho poad
    inicializace soust prohlee.
    \param parent Ukazatel na rodiovsk okno.
    \param name Jmno prohlee.
    \param embed Pznak vloen prohlee do svho rodie.
    \param build Pznak vytvoen okna prohlee. */
    SoQtFreeViewer(QWidget * parent, const char * const name, SbBool embed,
      SbBool build);
    /** Obecn callback udlost.
    Zpracuje poadavek na obslouen udlosti prohlee a vrt \p TRUE
    v ppad, e byla udlost obslouena. Ve skutenosti pouze zjist typ
    udlosti a obslouen ponech na pslun callback zazen.
    \param event Objekt udlosti.
    \return Pznak obslouen udlosti. */
    virtual SbBool processSoEvent(const SoEvent * const event);
    /** Callback udlost pohybu myi.
    Zpracuje udlost pohybu myi prohlee a vrt \p TRUE v ppad, e byla
    udlost obslouena.
    \param event Objekt udlosti.
    \return Pznak obslouen udlosti. */
    virtual SbBool processSoLocation2Event(const SoLocation2Event * const
      event);
    /** Callback udlost klvesnice.
    Zpracuje udlost stisku klvesy prohlee a vrt \p TRUE v ppad, e
    byla udlost obslouena.
    \param event Objekt udlosti.
    \return Pznak obslouen udlosti. */
    virtual SbBool processSoKeyboardEvent(const SoKeyboardEvent * const event);
    /** Callback udlost stisku tlatka myi.
    Zpracuje udlost stisku tlatka myi prohlee a vrt \p TRUE v ppad,
    e byla udlost obslouena.
    \param event Objekt udlosti.
    \return Pznak obslouen udlosti. */
    virtual SbBool processSoMouseButtonEvent(const SoMouseButtonEvent * const
      event);
    /* Datove polozky. */
    /// Cytlivost myi.
    float sensitivity;
    /// Pznak invertovan myi.
    SbBool invert;
  private:
    /** Spolen st konstruktor.
    Provede inicializace prohlee spolen jak pro privtn, tak pro veejn
    konstruktor.
    \param buildNow Pznak vytvoen okna prohlee. */
    void constructorCommon(SbBool buildNow);
    /** Vrt sted okna prohlee.
    Zjist a vrt sted okna prohlee v pixelech relativn od potku okna.
    \return Sted okna prohlee. */
    SbVec2s getCenter() const;
    /** Vrt pozici okna prohlee.
    Zjist a vrt pozici okna prohlee v pixelech relativn od potku rodie
    okna prohlee.
    \return Pozice okna prohlee. */
    SbVec2s getPosition() const;
    /** Nastav pozici kurzoru prohlee.
    Nastav pozici kurzoru prohlee na hodnotu \p position absolutn od
    potku obrazovky.
    \param position Nov pozice kurzoru prohlee. */
    void setCursorPosition(const SbVec2s position);
    /** Vrt pozici kurzoru prohlee.
    Vrt hodnotu pozice kurzoru prohlee absolutn od potku obrazovky.
    \return Pozice kurzoru prohlee.  */
    SbVec2s getCursorPosition();
};

#endif
