#ifndef PR_PROFILER_H
#define PR_PROFILER_H

///////////////////////////////////////////////////////////////////////////////
//  SoTerrain
///////////////////////////////////////////////////////////////////////////////
/// Tda profileru algoritm.
/// file PrProfiler.h
/// \author Radek Barto - xbarto33
/// \date 26.06.2005
///
/// Tato statick tda slou pro men doby strven vykonvnm rznch
/// st algoritm. Lze ji pout bu pmo nebo za pomoc maker. Nejprve je
/// nutno provst inicializaci tdy pomoc metody PrProfiler::initProfiler()
/// resp. makra PR_INIT_PROFILER(), poslze lze mit dobu strvenou v njakm
/// seku kdu pomoc PrProfiler::startProfile() a PrProfiler::stopProfile()
/// resp. PR_START_PROFILE() a PR_STOP_PROFILE(). Tyto pkazy maj tak svou
/// objektovou verzi. Nakonec je mono vytisknout vsledky men do souboru
/// pkazy PrProfiler::printResults() resp. PR_PRINT_RESULTS().
///////////////////////////////////////////////////////////////////////////////
//
// THIS SOFTWARE IS NOT COPYRIGHTED
//
// This source code is offered for use in the public domain.
//  You may use, modify or distribute it freely.
//
// This source code is distributed in the hope that it will be useful but
// WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
// DISCLAIMED.  This includes but is not limited to warranties of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// If you find the source code useful, authors will kindly welcome
// if you give them credit and keep their names with their source code.
//
///////////////////////////////////////////////////////////////////////////////

#include <cstdlib>
#include <vector>
#include <fstream>
#ifdef __GNUC__
  #include <sys/time.h>
#else
  #include <time.h>
#endif

#include <iostream>

#include <debug.h>

#ifdef PROFILE

/// Typ pro uloen asu - platformn zvisl.
#ifdef __GNUC__
  typedef unsigned long long PrTimeStamp;
#else
  typedef int64_t PrTimeStamp;
#endif

/** Jeden zznam men.
Tato tda slou pro ukldn jednotlivch zznam profilovn do pamti
a nsledn vytisknut do zadanho souboru. */
struct PrResult
{
  public:
    /** Konstruktor.
    Vytvo instanci tdy ::PrProfiler s nulovmi hodnotami. */
    PrResult();
    /** Konstruktor.
    Vytvo instanci tdy ::PrProfiler s hodnotami \p start, \p alg_id,
    \p object a \p time_stamp.
    \param start Pznak zatku profilovn seku kdu.
    \param alg_id ID algoritmu piazenho seku kdu.
    \param object Instance objektu psluejcho zznamu.
    \param time_stamp asov raztko zznamu. */
    PrResult(bool start, const int alg_id, const void * object,
      PrTimeStamp time_stamp);
    /// Pznak zatku profilovn seku kdu.
    bool start;
    /// ID algoritmu piazenho seku kdu.
    int alg_id;
    /// Instance objektu psluejcho zznamu.
    const void * object;
    /// asov raztko zznamu.
    PrTimeStamp time_stamp;
};

/** Tda profileru algoritm.
Tato statick tda slou pro men doby strven vykonvnm rznch
st algoritm. Lze ji pout bu pmo nebo za pomoc maker. Nejprve je
nutno provst inicializaci tdy pomoc metody PrProfiler::initProfiler()
resp. makra PR_INIT_PROFILER(), poslze lze mit dobu strvenou v njakm
seku kdu pomoc PrProfiler::startProfile() a PrProfiler::stopProfile()
resp. PR_START_PROFILE() a PR_STOP_PROFILE(). Tyto pkazy maj tak svou
objektovou verzi. Nakonec je mono vytisknout vsledky men do souboru
pkazy PrProfiler::printResults() resp. PR_PRINT_RESULTS(). */
class PrProfiler
{
  public:
    /* Metody. */
    /** Inicializace profileru.
    Zjist takt procesoru, inicializuje buffer pro zapisovn namench
    vsledk, inicializuje systm pro generovn ID algoritm. */
    static void initProfiler();
    /** Vytvo zznam zatku algoritmu.
    Spust men asu bhu algoritmu s ID  \p alg_id.
    \param alg_id ID sputnho algoritmu. */
    static void startProfile(const int alg_id);
    /** Vytvo zznam zatku algoritmu (objektov verze).
    Spust men asu bhu algoritmu s ID \p alg_id v objektu \p object.
    \param alg_id ID sputnho algoritmu.
    \param object Instance objektu algoritmu. */
    static void startProfile(const int alg_id, const void * object);
    /** Vytvo zznam konce algoritmu.
    Ukon men asu bhu agoritmu s ID \p alg_id.
    \param alg_id ID sputnho algoritmu. */
    static void stopProfile(const int alg_id);
    /** Vytvo zznam konce algoritmu (objektov verze).
    Ukon men asu bhu agoritmu s ID \p alg_id na objektu \p object.
    \param alg_id ID sputnho algoritmu.
    \param object Instance objektu algoritmu. */
    static void stopProfile(const int alg_id, const void * object);
    /** Vyprzdn buffer zznam men.
    Vyst vsledky men doby trvn algoritm. */
    static void clearBuffer();
    /** Vrt ID algoritmu.
    Piad nzvu algoritmu \p alg_name jedinen ID a vrat jej.
    \param alg_name Nzev algoritmu.
    \return ID algoritmu. */
    static const int getAlgId(const char * alg_name);
    /** Vrt nzev algoritmu.
    Vrt nzev algoritmu s ID \p alg_id. Pokud agoritmus s danm ID
    nenexistuje vrt \p NULL.
    \param alg_id ID algoritmu.
    \return Nzev algoritmu. */
    static const char * getAlgName(const int alg_id);
    /** Vrt poet asovch jednotek za mikrosekundu.
    Vrt poet takt asovho raztka, kter je ve struktue ::PrResult, za
    jednu mikrosekundu. Po vydlen asovho raztka touto hodnotou lze zskat
    jeho hodnotu v mikrosekundch.
    \return Poet asovch jednotek za mikrosekundu. */
    static const PrTimeStamp getTicksPerUsec();
    /** Vrt poet namench hodnot.
    Vrt poet namench hodnot v bufferu, kter byly doposud nameny.
    \return Poet namench hodnot. */
    static long getResultCount();
    /** Vrt vsledek men.
    Vrt namenou hodnotu na indexu \p index. Pekro-li index poet
    namench vsledk, vrt PrProfiler::NULL_RESULT. asov raztko vydlen
    hodnotou vrcenou z funkce PrProfiler::getTicksPerUsec() udv poet
    mikrosekund od potku men definovanho inicializac profileru.
    \param index Index vsledku men.
    \return Vsledek men. */
    static const PrResult & getResult(long index);
    /** Vytiskne vsledky men.
    Vytiskne vsledky na konec souboru \p filename. Pokud se tisk poda vrt
    \p true, jinak \p false.
    \param filename Soubor kam vsledky tisknout.
    \return Pznak spnho tisku vsledk. */
    static bool printResults(const char * filename);
    /* Konstanty. */
    /// Hodnota neplatnho ID.
    static const int NULL_ID;
    /// Vysledek men s przdnou hodnotou.
    static const PrResult NULL_RESULT;
  protected:
    /* Promenne. */
    /// Vektor vsledk men.
    static std::vector<PrResult> result_buffer;
    /// Vektor nzv algoritm.
    static std::vector<const char *> alg_names;
    /// Poet takt CPU za mikrosekundu.
    static PrTimeStamp ticks_per_usec;
    /// asov raztko pi zatku men.
    static PrTimeStamp start_time_stamp;
};

/* Makra pro snadnejsi pouzivani tridy PrProfiler. Jsou funkcni pouze tehdy,
je-li definovana promenna preprocesoru PROFILE. */
/** Ur zatek profilovn.
Vytvo zznam zatku profilovn algoritmu s nzvem \p name. */
#define PR_START_PROFILE(name) \
  static int name##_id = PrProfiler::getAlgId(#name); \
  PrProfiler::startProfile(name##_id)
/** Ur zatek profilovn (objektov verze).
Vytvo zznam zatku profilovn algoritmu s nzvem \p name patcho objektu
\p object. */
#define PR_START_OBJ_PROFILE(name, object) \
  static int name##_id = PrProfiler::getAlgId(#name); \
  PrProfiler::startProfile(name##_id, object)
/** Ur konec profilovn.
Vytvo zznam konce profilovn algoritmu s nzvem \p name. */
#define PR_STOP_PROFILE(name) PrProfiler::stopProfile(name##_id)
/** Ur konec profilovn (objektov verze).
Vytvo zznam konce profilovn algoritmu s nzvem \p name patcho objektu
\p object. */
#define PR_STOP_OBJ_PROFILE(name, object) \
  PrProfiler::stopProfile(name##_id, object)
/** Inicializace profileru.
Incializace celho profileru, je nutno zavolat ped jeho pouitm,
nejlpe na zatku programu. */
#define PR_INIT_PROFILER() PrProfiler::initProfiler()
/** Tisk vsledku profilovn.
Vytiskne vsledky profilovn na konec souboru \p filename ve formtu
definovanm v PrProfiler::printResults(). */
#define PR_PRINT_RESULTS(filename) PrProfiler::printResults(filename)

#else // PROFILE

struct PrResult;
class PrProfiler;

#define PR_START_PROFILE(name)
#define PR_START_OBJ_PROFILE(name, object)
#define PR_STOP_PROFILE(name)
#define PR_STOP_OBJ_PROFILE(name, object)
#define PR_INIT_PROFILER()
#define PR_PRINT_RESULTS(filename)

#endif // PROFILE
#endif // PR_PROFILER_H
