#ifndef SO_PROFILE_GROUP_H
#define SO_PROFILE_GROUP_H

///////////////////////////////////////////////////////////////////////////////
//  SoTerrain
///////////////////////////////////////////////////////////////////////////////
/// Tda roziujc tdu SoGroup o monosti profilovn.
/// \file SoProfileGroup.h
/// \author Radek Barto - xbarto33
/// \date 18.04.2006
///
/// Umonuje zmit dobu strvenou vykreslovnm uzl obsaench v tto skupin
/// za pomoc profileru ve statick tde ::PrProfiler.
///////////////////////////////////////////////////////////////////////////////
//
// THIS SOFTWARE IS NOT COPYRIGHTED
//
// This source code is offered for use in the public domain.
//  You may use, modify or distribute it freely.
//
// This source code is distributed in the hope that it will be useful but
// WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
// DISCLAIMED.  This includes but is not limited to warranties of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// If you find the source code useful, authors will kindly welcome
// if you give them credit and keep their names with their source code.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef PROFILE

#include <Inventor/nodes/SoGroup.h>
#include <Inventor/SbTime.h>

#include <profiler/PrProfiler.h>

/** Tda roziujc tdu \p ::SoGroup o monosti profilovn.
Umonuje zmit dobu strvenou vykreslovnm uzl obsaench v tto skupin
za pomoc profileru ve statick tde \p ::PrProfiler. */
class SoProfileGroup: public SoGroup
{
  SO_NODE_HEADER(SoProfileGroup);
  public:
    /* Metody. */
    /** Run-time inicializace tdy.
    Tuto metodu je teba zavolat ped vytvoenm jakkoliv instance tdy
    ::SoProfileGroup. */
    static void initClass();
    /** Konstruktor.
    Vytvo instanci tdy \p ::SoProfileGroup. */
    SoProfileGroup();
    /** Konstruktor.
    Vytvo instanci tdy \p ::SoProfileGroup s pedem odhadovanm potem
    potomk \p numChildren.
    \param numChildren Odhadovan poet potomk skupiny. */
    SoProfileGroup(int numChildren);
    /** Vykresleni potomk.
    Vykresli sv potomky a zm dobu strvenou tmto vykreslenm.
    \param action Objekt nesouc informace o grafu scny. */
    virtual void GLRender(SoGLRenderAction * action);
    /** Zjitn doby trvn poslednho vykreslen.
    Vrt dobu, kterou trvalo posledn vykreslen potomk.
    \return Doba poslednho vykreslovn. */
    virtual SbTime getLastRenderTime() const;
  protected:
    /* Datove polozky. */
    /// Poet jednotek asu profilovac knihovny za mikrosekundu.
    PrTimeStamp ticks_per_usec;
    /// Doba vykreslovn poslednho snmku.
    SbTime last_render_time;
  private:
    /* Metody. */
    /** Destruktor.
    Zprivatizovn, aby se zabrnilo ruen uzlu, nebo Inventor uvoluje pam
    ve vlastn reii. */
    virtual ~SoProfileGroup();
};

#else // PROFILE

#include <Inventor/nodes/SoGroup.h>
#include <Inventor/SbTime.h>

class SoProfileGroup: public SoGroup
{
  SO_NODE_HEADER(SoProfileGroup);
  public:
    static void initClass();
    SoProfileGroup();
    SoProfileGroup(int numChildren);
  private:
    virtual ~SoProfileGroup();
};

#endif // PROFILE
#endif // SO_PROFILE_SCENE_MANAGER_H
