#ifndef SO_PROFILE_SCENE_MANAGER_H
#define SO_PROFILE_SCENE_MANAGER_H

///////////////////////////////////////////////////////////////////////////////
//  SoTerrain
///////////////////////////////////////////////////////////////////////////////
/// Tda roziujc tdu SoSceneManager o monosti profilovn.
/// \file SoProfileSceneManager.h
/// \author Radek Barto - xbarto33
/// \date 05.07.2005
///
/// Tento manaer scny umouje zjistit jakou dobu strvil procesor potae
/// vykreslovnn scny. Pro pouit je teba nahradit vchoz manaer scny
/// tmto a inicializovat profilovac knihovnu ve td ::PrProfiler.
///////////////////////////////////////////////////////////////////////////////
//
// THIS SOFTWARE IS NOT COPYRIGHTED
//
// This source code is offered for use in the public domain.
//  You may use, modify or distribute it freely.
//
// This source code is distributed in the hope that it will be useful but
// WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
// DISCLAIMED.  This includes but is not limited to warranties of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// If you find the source code useful, authors will kindly welcome
// if you give them credit and keep their names with their source code.
//
///////////////////////////////////////////////////////////////////////////////

#include <Inventor/SoSceneManager.h>
#include <Inventor/SbTime.h>

#include <profiler/PrProfiler.h>

#ifdef PROFILE

/** Tda roziujc tdu SoSceneManager o monosti profilovn.
Tento manaer scny umouje zjistit jakou dobu strvil procesor potae
vykreslovnn scny. Pro pouit je teba nahradit vchoz manaer scny
tmto a inicializovat profilovac knihovnu ve td ::PrProfiler. */
class SoProfileSceneManager: public SoSceneManager
{
  public:
    /** Konstruktor.
    Nastav vychoz instance td ::SoGLRenderAction a ::SoHandleEventAction. */
    SoProfileSceneManager();
    /** Destruktor.
    Zru instanci tdy ::SoProfileSceneManager. */
    virtual ~SoProfileSceneManager();
    /** Vykresl graf sceny.
    M-li \p clearwindow hodnotu \p TRUE, nejdve vymae buffer pro vykreslovn.
    M-li \p clearzbuffer hodnotu \p TRUE, ped vykreslovnm vymae z-buffer.
    Oba tyto parametry by za normlnch okolnost mly bt \p TRUE, ale ve
    specilnch ppadech mohou bt nastaveny na \p FALSE kvli optimalizaci.
    (Nap. pi vykreslovn pomoc ar nen poteba pout z-buffer. */
    virtual void render(const SbBool clearwindow = TRUE,
      const SbBool clearzbuffer = TRUE);
    /** Zjitn doby trvn poslednho vykreslen.
    Vrt dobu, kterou trvalo posledn vykreslen grafu sceny. */
    virtual SbTime getLastRenderTime() const;
  protected:
    /// Poet jednotek asu profilovac knihovny za mikrosekundu.
    PrTimeStamp ticks_per_usec;
    /// Doba vykreslovn poslednho snmku.
    SbTime last_render_time;
};

#else

class SoProfileSceneManager: public SoSceneManager
{
  // nic
};

#endif // PROFILE
#endif // SO_PROFILE_SCENE_MANAGER_H
