#ifndef SB_ROAM_MERGE_QUEUE_H
#define SB_ROAM_MERGE_QUEUE_H

///////////////////////////////////////////////////////////////////////////////
//  SoTerrain
///////////////////////////////////////////////////////////////////////////////
/// Prioritn fronta diamant na spojen.
/// \file SbROAMMergeQueue.h
/// \author Radek Barto - xbarto33
/// \date 25.08.2005
///
/// Do tto prioritn fronty se vkldaj diamanty, kter lze spojit na dva
/// trojhelnky a tm dva trojhelnky z pvodnch ty z triangulace odstranit.
/// Toto se provd tehdy, pokud je teba triangulaci zjednoduit a to tak, e
/// se vyjme diamant s minimln prioritou, spoj se a vloi se zpt ppadn
/// vznikl nov diamanty.
///////////////////////////////////////////////////////////////////////////////
//
// THIS SOFTWARE IS NOT COPYRIGHTED
//
// This source code is offered for use in the public domain.
//  You may use, modify or distribute it freely.
//
// This source code is distributed in the hope that it will be useful but
// WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
// DISCLAIMED.  This includes but is not limited to warranties of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// If you find the source code useful, authors will kindly welcome
// if you give them credit and keep their names with their source code.
//
///////////////////////////////////////////////////////////////////////////////

#include <Inventor/SbHeap.h>

struct SbROAMSplitQueueTriangle;
struct SbROAMMergeQueueDiamond;

/** Prioritn fronta diamant na spojen algoritmu ROAM.
Do tto prioritn fronty se vkldaj diamanty, kter lze spojit na dva
trojhelnky a tm dva trojhelnky z pvodnch ty z triangulace odstranit.
Toto se provd tehdy, pokud je teba triangulaci zjednoduit a to tak, e
se vyjme diamant s minimln prioritou, spoj se a vloi se zpt ppadn
vznikl nov diamanty. */
class SbROAMMergeQueue : protected SbHeap
{
  public:
    /* Metody. */
    /** Konstruktor.
    Vytvo instanci tdy ::SbROAMMergeQueue o poten velikosti
    \p init_size poloek. Prioritn fronta se sama podle poteb zvtuje.
    \param init_size Poten velikost prioritn fronty trojhelnk
    na rozdlen.  */
    SbROAMMergeQueue(const int init_size = 4096);
    /** Destruktor.
    Zru instanci tdy ::SbROAMMergeQueue. */
    ~SbROAMMergeQueue();
    /** Vyprzdn prioritn frontu.
    Zavolnm tto metody dojde k vyprzdnn prioritn fronty diamant na
    spojen. */
    void emptyQueue();
    /** Vloen diamantu do fronty.
    Vlo diamant \p diamond do prioritn fronty diamant na spojen.
    \param diamond Diamant, kter se m do fronty vloit. */
    void add(SbROAMMergeQueueDiamond * diamond);
    /** Odstran diamant z fronty.
    Odstran diamant \p diamond z prioritn fronty diamant.
    \param diamond Diamant, kter se m z fronty odstranit. */
    void remove(SbROAMMergeQueueDiamond * diamond);
    /** Odstran diamant z fronty podle jeho trojhelnku.
    Najde ve front diamant na spojen diamant, jen obsahuje trojhelnk
    \p triangle, vyjme ho z fronty a vrt na nj ukazatel.
    \param triangle Trojhelnk, patc odstraovanmu diamantu.
    \return Ukazatel na odstrann diamant. */
    SbROAMMergeQueueDiamond * remove(SbROAMSplitQueueTriangle * triangle);
    /** Vyjme a vrt diamant z fronty.
    Odstran z prioritn fronty diamant na spojen trojhelnk s
    nejni prioritou a vrt ukazatel na nj.
    \return Ukazatel na diamant s nejni prioritou. */
    SbROAMMergeQueueDiamond * extractMin();
    /** Vrt diamant z fronty.
    Vrt ukazatel na diamant s nejni prioritou ve front. Diamant ve front
    nadle zstv.
    \return Ukazatel na diamant s nejni prioritou. */
    SbROAMMergeQueueDiamond * getMin();
    /** Vrt diamant z fronty podle indexu.
    Vrt ukazatel na diamant na indexu \p index v prioritn front diamant.
    \return Ukazatel na trojhelnk na indexu \p index. */
    SbROAMMergeQueueDiamond * operator[](const int index);
    using SbHeap::size;
    using SbHeap::newWeight;
    using SbHeap::buildHeap;
  protected:
    /* Metody. */
    /** Vrt prioritu diamantu.
    Vrt prioritu diamantu \p diamond v prioritn front diamant na spojen.
    \param diamond Diamant, jeho priorita se m zskat.
    \return Priorita diamantu. */
    static float getDiamondPriority(void * diamond);
    /** Vrt index diamantu.
    Vrt index diamantu \p diamant v prioritn front diamant na spojen.
    \param diamond Diamant, jeho index se m zskat.
    \return Index diamantu. */
    static int getDiamondIndex(void * diamond);
    /** Nastav index diamantu.
    Nastav index diamantu \p diamond v prioritn front diamant na hodnotu
    \p index.
    \param diamond Diamant, u kterho se m index nastavit.
    \param index Nov index diamantu. */
    static void setDiamondIndex(void * diamond, int index);
    /* Datove polozky. */
    /// Struktura callback obsahujc metody pro prci s diamanty.
    static SbHeapFuncs heap_funcs;
  private:
};

#endif
