#ifndef SB_ROAM_SPLIT_QUEUE_H
#define SB_ROAM_SPLIT_QUEUE_H

///////////////////////////////////////////////////////////////////////////////
//  SoTerrain
///////////////////////////////////////////////////////////////////////////////
/// Prioritn fronta trojhelnk na rozdlen.
/// \file SbROAMSplitQueue.h
/// \author Radek Barto - xbarto33
/// \date 25.08.2005
///
/// Do tto prioritn fronty se vkldaj trojhelnky, kter je teba zaadit
/// do triangulace. Pokud je teba triangulaci dle zjemovat, vyjme se z fronty
/// trojhelnk s maximln prioritou a vlo se zpt jeho potomci.
///////////////////////////////////////////////////////////////////////////////
//
// THIS SOFTWARE IS NOT COPYRIGHTED
//
// This source code is offered for use in the public domain.
//  You may use, modify or distribute it freely.
//
// This source code is distributed in the hope that it will be useful but
// WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
// DISCLAIMED.  This includes but is not limited to warranties of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// If you find the source code useful, authors will kindly welcome
// if you give them credit and keep their names with their source code.
//
///////////////////////////////////////////////////////////////////////////////

#include <Inventor/SbHeap.h>

struct SbROAMSplitQueueTriangle;

/** Prioritn fronta trojhelnk na rozdlen algoritmu ROAM.
Do tto prioritn fronty se vkldaj trojhelnky, kter je teba zaadit
do triangulace. Pokud je teba triangulaci dle zjemovat, vyjme se z fronty
trojhelnk s maximln prioritou a vlo se zpt jeho potomci. */
class SbROAMSplitQueue : protected SbHeap
{
  public:
    /* Metody. */
    /** Konstruktor.
    Vytvo instanci tdy ::SbROAMSplitQueue o poten velikosti
    \p init_size poloek. Prioritn fronta se sama podle poteb zvtuje.
    \param init_size Poten velikost prioritn fronty trojhelnk
    na rozdlen.  */
    SbROAMSplitQueue(const int init_size = 1024);
    /** Destruktor.
    Zru instanci tdy ::SbROAMSplitQueue. */
    ~SbROAMSplitQueue();
    /** Vyprzdn prioritn frontu.
    Zavolnm tto metody dojde k vyprzdnn prioritn fronty trojhelnk
    na rozdlen. */
    void emptyQueue();
    /** Vlo trojhelnk do fronty.
    Vlo trojhelnk binrnho stromu trojhelnk \p triangle do prioritn
    fronty trojhelnk na rozdlen.
    \param triangle Trojhelnk, kter se m do fronty vloit. */
    void add(SbROAMSplitQueueTriangle * triangle);
    /** Odstran trojhelnk z fronty.
    Odstran trojhelnk \p triangle z prioritn fronty trojhelnk.
    \param triangle Trojhelnk, kter se m z fronty odstranit. */
    void remove(SbROAMSplitQueueTriangle * triangle);
    /** Vyjme a vrt trojhelnk z fronty.
    Odstran z prioritn fronty trojhelnk na rozdlen trojhelnk s
    nejvy prioritou a vrt ukazatel na nj.
    \return Ukazatel na trojhelnk s nejvy prioritou. */
    SbROAMSplitQueueTriangle * extractMax();
    /** Vrt trojhelnk z fronty.
    Vrt ukazatel na trojhelnk s nejvy prioritou ve front. Trojhelnk
    ve front nadle zstv.
    \return Ukazatel na trojhelnk s nejvy prioritou. */
    SbROAMSplitQueueTriangle * getMax();
    /** Vrt trojhelnk z fronty podle indexu.
    Vrt ukazatel na trojhelnk na indexu \p index v prioritn front
    trojhelnk.
    \return Ukazatel na trojhelnk na indexu \p index. */
    SbROAMSplitQueueTriangle * operator[](const int index);
    /* Importovane metody. */
    using SbHeap::size;
    using SbHeap::newWeight;
    using SbHeap::buildHeap;
 protected:
    /* Metody. */
    /** Vrt prioritu trojhelnku.
    Vrt prioritu trojhelnku \p triangle v prioritn front trojhelnk
    na rozdlen.
    \param triangle Trojhelnk, jeho priorita se m zskat.
    \return Priorita trojhelnku. */
    static float getTrianglePriority(void * triangle);
    /** Vrt index trojhelnku.
    Vrt index trojhelnku \p triangle v prioritn front trojhelnk
    na rozdlen.
    \param triangle Trojhelnk, jeho index se m zskat.
    \return Index trojhelnku. */
    static int getTriangleIndex(void * triangle);
    /** Nastav index trojhelnku.
    Nastav index trojhelnku \p triangle v prioritn front trojhelnk
    na hodnotu \p index.
    \param triangle Trojhelnk, u kterho se m index nastavit.
    \param index Nov index trojhelnku. */
    static void setTriangleIndex(void * triangle, int index);
    /* Datove polozky. */
    /// Struktura callback obsahujc metody pro prci s trojhelnky.
    static SbHeapFuncs heap_funcs;
};

#endif
