#!/bin/env python
# Name:    Creates graphs from performance test results.
# Author:  Radek Barton
# E-mail:  blachex@post.cz
# Date:    7. 4. 2005
# Version: 0.1

import os, os.path, sys, pickle

from performance_values import *

###############################################################################

def read_value(file_names, algorithm, position):
  command = 'scripts/prstat.py -f %s -a %s -p' % (file_names, algorithm)
  try:
    output = os.popen(command, 'r')
    for line in output.readlines():
      value = float(line.split(':')[position])
  except:
    return 0.0
  return value

###############################################################################

try:
  cache_file = open('cache.dat', 'r')
  results = pickle.load(cache_file)
  cache_file.close()
except:
  results = {}

# Read FPS results from files
for algorithm in algorithms:
  for landscape in landscapes:
    for animation_time in animation_times:
      profile_file = ''
      if (algorithm == 'roam'):
        for triangle_count in triangle_counts:
          for pixel_error in pixel_errors:
            key = (algorithm, landscape[0], animation_time, triangle_count,
              pixel_error)
            if not results.has_key(key):
              file_names = ''
              for I in xrange(10):
                file_name = 'results/%s-%s-%s-%s-%s-%s.txt' % (algorithm,
                landscape[0], animation_time, triangle_count, pixel_error, I)
                file_names = file_names + ':' + file_name
              preprocess_value = read_value(file_names[1:], 'preprocess', 6) / 1000.0
              fps_value = read_value(file_names[1:], 'SoProfileGroup_GLRender', 3) \
                * 1000.0 / float(animation_time)
              results[key] = (fps_value, preprocess_value)
      elif (algorithm == 'geomipmapping'):
        for tile_size in tile_sizes:
          for pixel_error in pixel_errors:
            key = (algorithm, landscape[0], animation_time, tile_size,
              pixel_error)
            if not results.has_key(key):
              file_names = ''
              for I in xrange(10):
                file_name = 'results/%s-%s-%s-%s-%s-%s.txt' % (algorithm,
                  landscape[0], animation_time, tile_size, pixel_error, I)
                file_names = file_names + ':' + file_name
              preprocess_value = read_value(file_names[1:], 'preprocess', 6) / 1000.0
              fps_value = read_value(file_names[1:], 'SoProfileGroup_GLRender', 3) \
                * 1000.0 / float(animation_time)
              results[key] = (fps_value, preprocess_value)
      else:
        key = (algorithm, landscape[0], animation_time)
        if not results.has_key(key):
          file_names = ''
          for I in xrange(10):
            file_name = 'results/%s-%s-%s-%s.txt' % (algorithm, landscape[0],
              animation_time, I)
            file_names = file_names + ':' + file_name
          results[key] = read_value(file_names[1:], 'SoProfileGroup_GLRender', 3) \
            * 1000.0 / float(animation_time)

try:
  cache_file = open('cache.dat', 'w+')
  pickle.dump(results, cache_file)
  cache_file.close()
except:
  pass

# Generate graph about ROAM algorithm
for landscape in landscapes:
  file = open('tmp.dat', 'w+')
  for animation_time in ('10000', '100000'):
    for triangle_count in triangle_counts:
      for pixel_error in pixel_errors:
        key = ('roam', landscape[0], animation_time, triangle_count,
          pixel_error)
        if results.has_key(key):
          value = results[key][0]
        else:
          value = 0.0
        file.write('%s %s\n' % (pixel_error, value))
      file.write('\n\n')
  file.close()
  os.system('gnuplot plots/roam-%s.plot > images/graphs/roam-%s.svg' % \
    (landscape[0], landscape[0]))
  os.system('rm tmp.dat')

for landscape in landscapes:
  file = open('tmp.dat', 'w+')
  for animation_time in ('10000', '100000'):
    for pixel_error in pixel_errors:
      for triangle_count in triangle_counts:
        key = ('roam', landscape[0], animation_time, triangle_count,
          pixel_error)
        if results.has_key(key):
          value = results[key][0]
        else:
          value = 0.0
        file.write('%s %s\n' % (triangle_count, value))
      file.write('\n\n')
  file.close()
  os.system('gnuplot plots/roam2-%s.plot > images/graphs/roam2-%s.svg' % \
    (landscape[0], landscape[0]))
  os.system('rm tmp.dat')

# Generate graph about Geo Mip-Mapping algorithm
for landscape in landscapes:
  file = open('tmp.dat', 'w+')
  for animation_time in ('10000', '100000'):
    for pixel_error in pixel_errors:
      for tile_size in tile_sizes:
        key = ('geomipmapping', landscape[0], animation_time, tile_size,
          pixel_error)
        if results.has_key(key):
          value = results[key][0]
        else:
          value = 0.0
        file.write('%s %s\n' % (tile_size, value))
      file.write('\n\n')
  file.close()
  os.system('gnuplot plots/geomipmapping-%s.plot > images/graphs/geomipmapping-%s.svg' \
    % (landscape[0], landscape[0]))
  os.system('rm tmp.dat')

for landscape in landscapes:
  file = open('tmp.dat', 'w+')
  for animation_time in ('10000', '100000'):
    for tile_size in tile_sizes:
      for pixel_error in pixel_errors:
        key = ('geomipmapping', landscape[0], animation_time, tile_size,
          pixel_error)
        if results.has_key(key):
          value = results[key][0]
        else:
          value = 0.0
        file.write('%s %s\n' % (pixel_error, value))
      file.write('\n\n')
  file.close()
  os.system('gnuplot plots/geomipmapping2-%s.plot > images/graphs/geomipmapping2-%s.svg' \
    % (landscape[0], landscape[0]))
  os.system('rm tmp.dat')

file = open('tmp.dat', 'w+')
for algorithm in algorithms:
  for landscape in landscapes:
    if algorithm == 'roam':
      key = ('roam', landscape[0], '100000', '5000', '4')
    elif algorithm == 'geomipmapping':
      key = ('geomipmapping', landscape[0], '100000', '33', '4')
    else:
      key = ('brutalforce', landscape[0], '100000')
    if results.has_key(key):
      if algorithm == 'brutalforce':
        value = results[key]
      else:
        value = results[key][0]
    else:
      value = 0.0
    file.write('%s %s\n' % (landscape[3], value))
  file.write('\n\n')
file.close()
os.system('gnuplot plots/heightmap.plot > images/graphs/heightmap.svg')
os.system('rm tmp.dat')

file = open('tmp.dat', 'w+')
for algorithm in ('roam', 'geomipmapping'):
  for landscape in landscapes:
    if algorithm == 'roam':
      key = ('roam', landscape[0], '10000', '7000', '4')
    else:
      key = ('geomipmapping', landscape[0], '10000', '33', '4')
    if results.has_key(key):
      value = results[key][1]
    else:
      value = 0.0
    file.write('%s %s\n' % (landscape[3], value))
  file.write('\n\n')
file.close()
os.system('gnuplot plots/preprocessing.plot > images/graphs/preprocessing.svg')
os.system('rm tmp.dat')