#include <Inventor/SbBasic.h>
#include <Inventor/SbLinear.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoTexture2.h>
#include <Inventor/nodes/SoTextureCoordinate2.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoNormal.h>
#include <Inventor/nodes/SoNormalBinding.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/nodes/SoIndexedTriangleStripSet.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/sensors/SoTimerSensor.h>

#include <Inventor/Qt/SoQt.h>
#include <Inventor/Qt/viewers/SoQtExaminerViewer.h>

#include <iostream>
#include <cmath>

#include <simage.h>

#include <roam/SoSimpleROAMTerrain.h>
#include <geomipmapping/SoSimpleGeoMipmapTerrain.h>
#include <profiler/PrProfiler.h>
#include <profiler/SoProfileGroup.h>
#include <profiler/SoProfileSceneManager.h>
#include <SoQtFreeViewer.h>
#include <utils.h>

#if defined(__WIN32__) || defined(_WIN32)
  #pragma comment(linker, "/entry:\"mainCRTStartup\"")
  #pragma comment(linker, "/subsystem:\"console\"")
#endif

enum
{
  ID_ALG_BRUAL_FORCE = 0,
  ID_ALG_ROAM = 1,
  ID_ALG_GEO_MIPMAP = 2
};

int algorithm = ID_ALG_ROAM;
float animation_time = 30.0f;
float frame_time = 0.04f;
SbBool is_synchronize = FALSE;

/* Obsluha stisku tlacitek vykreslovaneho terenu. */
void terrainCallback(void * userData, SoEventCallback * eventCB)
{
  const SoKeyboardEvent * event =
    reinterpret_cast<const SoKeyboardEvent *>(eventCB->getEvent());

  if (SO_KEY_PRESS_EVENT(event, ENTER))
  {
    if (algorithm == ID_ALG_ROAM)
    {
      SoSimpleROAMTerrain * terrain = reinterpret_cast<SoSimpleROAMTerrain *>
        (userData);
      terrain->freeze.setValue(!terrain->freeze.getValue());
    }
    else
    {
      SoSimpleGeoMipmapTerrain * terrain = reinterpret_cast<SoSimpleGeoMipmapTerrain *>
        (userData);
      terrain->freeze.setValue(!terrain->freeze.getValue());
    }
    eventCB->setHandled();
  }
  else if (SO_KEY_PRESS_EVENT(event, PAD_ADD))
  {
    if (algorithm == ID_ALG_ROAM)
    {
      SoSimpleROAMTerrain * terrain = reinterpret_cast<SoSimpleROAMTerrain *>
        (userData);
      terrain->pixelError.setValue(terrain->pixelError.getValue() + 1);
    }
    else
    {
      SoSimpleGeoMipmapTerrain * terrain = reinterpret_cast<SoSimpleGeoMipmapTerrain *>
        (userData);
      terrain->pixelError.setValue(terrain->pixelError.getValue() + 1);
    }
    eventCB->setHandled();
  }
  else if (SO_KEY_PRESS_EVENT(event, PAD_SUBTRACT))
  {
    if (algorithm == ID_ALG_ROAM)
    {
      SoSimpleROAMTerrain * terrain = reinterpret_cast<SoSimpleROAMTerrain *>
        (userData);
      if (terrain->pixelError.getValue() > 1)
      {
        terrain->pixelError.setValue(terrain->pixelError.getValue() - 1);
      }
    }
    else
    {
      SoSimpleGeoMipmapTerrain * terrain = reinterpret_cast<SoSimpleGeoMipmapTerrain *>
        (userData);
      if (terrain->pixelError.getValue() > 1)
      {
        terrain->pixelError.setValue(terrain->pixelError.getValue() - 1);
      }
    }
    eventCB->setHandled();
  }
  else if (SO_KEY_PRESS_EVENT(event, F))
  {
    if (algorithm == ID_ALG_ROAM)
    {
      SoSimpleROAMTerrain * terrain = reinterpret_cast<SoSimpleROAMTerrain *>
        (userData);
      terrain->frustrumCulling.setValue(!terrain->frustrumCulling.getValue());
    }
    else
    {
      SoSimpleGeoMipmapTerrain * terrain = reinterpret_cast<SoSimpleGeoMipmapTerrain *>
        (userData);
      terrain->frustrumCulling.setValue(!terrain->frustrumCulling.getValue());
    }
    eventCB->setHandled();
  }
}

void styleCallback(void * userData, SoEventCallback * eventCB)
{
  SoDrawStyle * style = reinterpret_cast<SoDrawStyle *>(userData);
  const SoKeyboardEvent * event =
    reinterpret_cast<const SoKeyboardEvent *>(eventCB->getEvent());

  if (SO_KEY_PRESS_EVENT(event, L))
  {
    if (style->style.getValue() == SoDrawStyle::FILLED)
    {
      style->style.setValue(SoDrawStyle::LINES);
    }
    else
    {
      style->style.setValue(SoDrawStyle::FILLED);
    }
  }
}

/* Callback casovace ovladajiciho animaci kamery. */
void cameraTimerCallback(void * userData, SoSensor * sensor)
{
  /* Konstanty pro animaci kamery. */
  static const int CAMERA_POINTS_COUNT = 10;
  static const SbVec3f camera_positions[CAMERA_POINTS_COUNT] =
    {SbVec3f(0.0f, 0.1f, 0.03f), SbVec3f(0.6f, 0.1f, 0.03f),
     SbVec3f(0.6f, 0.1f, 0.03f), SbVec3f(0.6f, 0.6f, 0.03f),
     SbVec3f(0.6f, 0.6f, 0.03f), SbVec3f(0.2f, 0.6f, 0.03f),
     SbVec3f(0.2f, 0.6f, 0.03f), SbVec3f(0.6f, 0.6f, 0.03f),
     SbVec3f(0.6f, 0.6f, 0.03f), SbVec3f(0.6f, 0.1f, 0.03f)};
  static const SbRotation camera_orientations[CAMERA_POINTS_COUNT] =
    {SbRotation(SbVec3f(1.0f, 0.0f, 0.0f), static_cast<float>(M_PI) / 2.0f) *
       SbRotation(SbVec3f(0.0f, 0.0f, 1.0f), -static_cast<float>(M_PI) / 2.0f),
     SbRotation(SbVec3f(1.0f, 0.0f, 0.0f), static_cast<float>(M_PI) / 2.0f) *
       SbRotation(SbVec3f(0.0f, 0.0f, 1.0f), -static_cast<float>(M_PI) / 2.0f),
     SbRotation(SbVec3f(1.0f, 0.0f, 0.0f), static_cast<float>(M_PI) / 2.0f) *
       SbRotation(SbVec3f(0.0f, 0.0f, 1.0f), 0.0f),
     SbRotation(SbVec3f(1.0f, 0.0f, 0.0f), static_cast<float>(M_PI) / 2.0f) *
       SbRotation(SbVec3f(0.0f, 0.0f, 1.0f), 0.0f),
     SbRotation(SbVec3f(1.0f, 0.0f, 0.0f), static_cast<float>(M_PI) / 2.0f) *
       SbRotation(SbVec3f(0.0f, 0.0f, 1.0f), static_cast<float>(M_PI) / 2.0f),
     SbRotation(SbVec3f(1.0f, 0.0f, 0.0f), static_cast<float>(M_PI) / 2.0f) *
       SbRotation(SbVec3f(0.0f, 0.0f, 1.0f), static_cast<float>(M_PI) / 2.0f),
     SbRotation(SbVec3f(1.0f, 0.0f, 0.0f), static_cast<float>(M_PI) / 2.0f) *
       SbRotation(SbVec3f(0.0f, 0.0f, 1.0f), -static_cast<float>(M_PI) / 2.0f),
     SbRotation(SbVec3f(1.0f, 0.0f, 0.0f), static_cast<float>(M_PI) / 2.0f) *
       SbRotation(SbVec3f(0.0f, 0.0f, 1.0f), -static_cast<float>(M_PI) / 2.0f),
     SbRotation(SbVec3f(1.0f, 0.0f, 0.0f), static_cast<float>(M_PI) / 2.0f) *
       SbRotation(SbVec3f(0.0f, 0.0f, 1.0f), -static_cast<float>(M_PI)),
     SbRotation(SbVec3f(1.0f, 0.0f, 0.0f), static_cast<float>(M_PI) / 2.0f) *
       SbRotation(SbVec3f(0.0f, 0.0f, 1.0f), -static_cast<float>(M_PI))};

  static int frame = 0;
  static SbTime start_time = SbTime::zero();
  SoPerspectiveCamera * camera = reinterpret_cast<SoPerspectiveCamera *>
    (userData);

  if (frame == 0)
  {
    camera->position = camera_positions[0];
    camera->orientation = camera_orientations[0];
  }
  else
  {
    if (frame == 1)
    {
      start_time = SbTime::getTimeOfDay();
    }

    float position = 0.0;
    if (is_synchronize)
    {
      position = ((SbTime::getTimeOfDay() - start_time).getValue() / animation_time)
        * CAMERA_POINTS_COUNT;
    }
    else
    {
      position = ((frame * frame_time) / animation_time) * CAMERA_POINTS_COUNT;
    }

    int point = static_cast<int>(position);
    float ratio = position - point;

    if (point >= CAMERA_POINTS_COUNT - 1)
    {
      SoQt::exitMainLoop();
    }

    camera->position = (1.0f - ratio) * camera_positions[point] +
      ratio * camera_positions[point + 1];
    SbRotation orientation_1 = camera_orientations[point];
    SbRotation orientation_2 = camera_orientations[point + 1];
    camera->orientation = SbRotation::slerp(orientation_1, orientation_2, ratio);
  }

  frame++;
}

void renderCallback(void * _render_area, SoSceneManager * scene_manager)
{
  /* Prekresleni prohlizece pri pouziti vlastniho manazeru sceny. */
  SoQtRenderArea * render_area = reinterpret_cast<SoQtRenderArea *>(_render_area);
  render_area->render();
}

void help()
{
  std::cout << "Pouziti: SoTerrainTest -h heightmap [-t texture] [-p profile_file] "
    "[-a algorithm] [-A animation_time] [-F frame_time] [-e pixel_error] "
    "[-r triangle_count] [-g tile_size] [-f] [-c] [-v] [-s]" << std::endl;
  std::cout << "\t-h heightmap\t\tSoubor se vstupni vyskovou mapou." <<
    std::endl;
  std::cout << "\t-t texture\t\tSoubor s texturou terenu." << std::endl;
  std::cout << "\t-p profile_file\t\tSoubor pro ulozeni vysledku profilovani."
    << std::endl;
  std::cout << "\t-a algorithm\t\tVyber algoritmu pro zobrazeni terenu." <<
    std::endl;
  std::cout << "\t\tbrutalforce\t\tVykresleni terenu hrubou silou." << std::endl;
  std::cout << "\t\troam\t\t\tVykresleni terenu algoritmem ROAM." << std::endl;
  std::cout << "\t\tgeomipmapping\t\tVykresleni terenu algoritmem Geo Mip-Mapping."
    << std::endl;
  std::cout << "\t-A animation_time\tDelka animace v milisekundach." <<
    std::endl;
  std::cout << "\t-F frame_time\t\tDoba trvani jednoho snimku v milisekundach."
    << std::endl;
  std::cout << "\t-e pixel_error\t\tNastaveni chyby zobrazeni algoritmu." <<
    std::endl;
  std::cout << "\t-r triangle_count\tNastaveni poctu zobrazovanych trojuhelniku."
    << std::endl;
  std::cout << "\t-g tile_size\t\tNastaveni velikosti dlazdice." << std::endl;
  std::cout << "\t-f\t\t\tSpusteni programu na cele obrazovce." << std::endl;
  std::cout << "\t-c\t\t\tZapnuti orezavani pohledovym telesem." << std::endl;
  std::cout << "\t-v\t\t\tSpusteni animace po startu programu." << std::endl;
  std::cout << "\t-s\t\t\tZapnuti synchronizace snimku animace." << std::endl;
}

int main(int argc, char * argv[])
{
  /* Parametry programu a jejich vchoz hodnoty. */
  char * heightmap_name = NULL;
  char * texture_name = NULL;
  char * profile_name = "profile.txt";
  int triangle_count = 10000;
  int tile_size = 33;
  int pixel_error = 3;
  SbBool is_animation = FALSE;
  SbBool is_full_screen = FALSE;
  SbBool is_frustrum_culling = TRUE;

  /* Zjisteni parametru programu. */
  int command = 0;
  while ((command = getopt(argc, argv, "h:t:p:a:A:F:e:r:g:fcvs")) != -1)
  {
    switch (command)
    {
      /* Vyskova mapa. */
      case 'h':
      {
        heightmap_name = optarg;
      }
      break;
      /* Textura. */
      case 't':
      {
        texture_name = optarg;
      }
      break;
      /* Soubor pro ulozeni vysledku profilovani. */
      case 'p':
      {
        profile_name = optarg;
      }
      break;
      /* Algoritmus. */
      case 'a':
      {
        if (!strcmp(optarg, "roam"))
        {
          algorithm = ID_ALG_ROAM;
        }
        else if (!strcmp(optarg, "geomipmapping"))
        {
          algorithm = ID_ALG_GEO_MIPMAP;
        }
        else if (!strcmp(optarg, "brutalforce"))
        {
          algorithm = ID_ALG_BRUAL_FORCE;
        }
      }
      break;
      /* Doba animace. */
      case 'A':
      {
        int tmp = 0;
        sscanf(optarg, "%d", &tmp);
        animation_time = tmp * 0.001f;
      }
      break;
      /* Doba animace. */
      case 'F':
      {
        int tmp = 0;
        sscanf(optarg, "%d", &tmp);
        frame_time = tmp * 0.001f;
      }
      break;
      /* Zobrazovaci chyba v pixelech. */
      case 'e':
      {
        sscanf(optarg, "%d", &pixel_error);
      }
      break;
      /* Pocet trojuhelniku. */
      case 'r':
      {
        sscanf(optarg, "%d", &triangle_count);
      }
      break;
      /* Velikost dlazdice. */
      case 'g':
      {
        sscanf(optarg, "%d", &tile_size);
      }
      break;
      /* Fullscreen. */
      case 'f':
      {
        is_full_screen = TRUE;
      }
      break;
      /* Orezavani pohledovym telesem. */
      case 'c':
      {
        is_frustrum_culling = FALSE;
      }
      break;
      /* Spusteni animace. */
      case 'v':
      {
        is_animation = TRUE;
      }
      break;
      /* Spusteni animace. */
      case 's':
      {
        is_synchronize = TRUE;
      }
      break;
      case '?':
      {
        std::cout << "Nezamy parametr programu!" << std::endl;
        help();
        exit(1);
      }
      break;
    }
  }

  /* Kontrola povinneho parametru. */
  if (heightmap_name == NULL)
  {
    std::cout << "Nebyla zadana vstupni vyskova mapa!" << std::endl;
    help();
    exit(1);
  }

  /* Nacteni vyskove mapy. */
  int width = 0;
  int height = 0;
  int components = 0;
  unsigned char * heightmap = simage_read_image(heightmap_name, &width,
    &height, &components);
  if (heightmap == NULL)
  {
    std::cout << "Chyba pri nacitani vyskove mapy " << heightmap_name << "!"
      << std::endl;
    exit(1);
  }

  PR_INIT_PROFILER();

  /* Nastaveni promennych prostredi. */
  putenv("IV_SEPARATOR_MAX_CACHES=0");
  putenv("COIN_SHOW_FPS_COUNTER=1");
  putenv("COIN_AUTO_CACHING=0");

  /* Vytvoreni okna. */
  QWidget * window;
  if ((window = SoQt::init(argc, argv, "ROAM Test")) == NULL)
  {
    exit(1);
  }

  /* Inicializace vlastnich Inventorovskych trid. */
  SoSimpleROAMTerrain::initClass();
  SoSimpleGeoMipmapTerrain::initClass();
  SoProfileGroup::initClass();

  /* Vytvoreni objektu grafu sceny */
  SoProfileGroup * root = new SoProfileGroup();
  SoEventCallback * style_callback = new SoEventCallback();
  SoPerspectiveCamera * camera = new SoPerspectiveCamera();
  SoDrawStyle * style = new SoDrawStyle();
  SoDirectionalLight * light = new SoDirectionalLight();
  SoSeparator * separator = new SoSeparator();
  SoEventCallback * terrain_callback = new SoEventCallback();
  SoTexture2 * texture = new SoTexture2();
  SoTextureCoordinate2 * texture_coords = new SoTextureCoordinate2();
  SoCoordinate3 * coords = new SoCoordinate3();
  SoNormal * normals = new SoNormal();
  SoNormalBinding * normal_binding = new SoNormalBinding();

  /* Nastaveni jejich vlastnosti. */
  style_callback->addEventCallback(SoKeyboardEvent::getClassTypeId(),
    styleCallback, style);
  light->direction.setValue(0.5f, 0.5f, -1.0f);
  texture->filename.setValue(texture_name);
  coords->point.setNum(width * height);
  texture_coords->point.setNum(width * height);
  normals->vector.setNum(width * height);
  normal_binding->value.setValue(SoNormalBinding::PER_VERTEX_INDEXED);

  /* Vytvoreni heightmapy. */
  SbVec3f * points = coords->point.startEditing();
  SbVec2f * texture_points = texture_coords->point.startEditing();
  SbVec3f * normal_points = normals->vector.startEditing();
  for (int I = 0; I < width * height; ++I)
  {
    float x = float(I % width) / float(width);
    float y = float(I / width) / float(height);
    points[I] = SbVec3f(x, y, heightmap[I] * 0.0002f);
    texture_points[I] = SbVec2f(x, y);
  }

  /* Vypocet normal ve vnitrku vyskove mapy. */
  for (int Y = 1; Y < (height - 1); ++Y)
  {
    for (int X = 1; X < (width - 1); ++X)
    {
      int index = Y * width + X;
      SbVec3f normal = SbVec3f(0.0f, 0.0f, 0.0f);

      normal += (points[index - 1] - points[index]).cross(points[index
        - width] - points[index]);
      normal += (points[index - width] - points[index]).cross(points[index
        - width + 1] - points[index]);
      normal += (points[index - width + 1] - points[index]).cross(points[index
        + 1] - points[index]);
      normal += (points[index + 1] - points[index]).cross(points[index
        + width] - points[index]);
      normal += (points[index + width] - points[index]).cross(points[index
        + width - 1] - points[index]);
      normal += (points[index + width - 1] - points[index]).cross(points[index
        - 1] - points[index]);
      normal.normalize();
      normal_points[index] = normal;
    }
  }

  /* Vypocet normal na hornim a dolnim okraji. */
  for (int X = 1; X < (width - 1); ++X)
  {
    int index_1 = X;
    int index_2 = (height - 1) * width + X;
    SbVec3f normal_1 = SbVec3f(0.0f, 0.0f, 0.0f);
    SbVec3f normal_2 = SbVec3f(0.0f, 0.0f, 0.0f);

    /* Horni okraj. */
    normal_1 += (points[index_1 + 1] - points[index_1]).cross(points[index_1
      + width] - points[index_1]);
    normal_1 += (points[index_1 + width] - points[index_1]).cross(points[index_1
      + width - 1] - points[index_1]);
    normal_1 += (points[index_1 + width - 1] - points[index_1]).cross(points[index_1
      - 1] - points[index_1]);

    /* Dolni okraj. */
    normal_2 += (points[index_2 - 1] - points[index_2]).cross(points[index_2
      - width] - points[index_2]);
    normal_2 += (points[index_2 - width] - points[index_2]).cross(points[index_2
      - width + 1] - points[index_2]);
    normal_2 += (points[index_2 - width + 1] - points[index_2]).cross(points[index_2
      + 1] - points[index_2]);

    normal_1.normalize();
    normal_2.normalize();
    normal_points[index_1] = normal_1;
    normal_points[index_2] = normal_2;
  }

  /* Vypocet normal v levem a pravem okraji. */
  for (int Y2 = 1; Y2 < (height - 1); ++Y2)
  {
    int index_1 = Y2 * width;
    int index_2 = index_1 + width - 1;
    SbVec3f normal_1 = SbVec3f(0.0f, 0.0f, 0.0f);
    SbVec3f normal_2 = SbVec3f(0.0f, 0.0f, 0.0f);

    /* Levy okraj. */
    normal_1 += (points[index_1 - width] - points[index_1]).cross(points[index_1
      - width + 1] - points[index_1]);
    normal_1 += (points[index_1 - width + 1] - points[index_1]).cross(points[index_1
      + 1] - points[index_1]);
    normal_1 += (points[index_1 + 1] - points[index_1]).cross(points[index_1
      + width] - points[index_1]);

    /* Pravy okraj. */
    normal_2 += (points[index_2 - 1] - points[index_2]).cross(points[index_2
      - width] - points[index_2]);
    normal_2 += (points[index_2 + width] - points[index_2]).cross(points[index_2
      + width - 1] - points[index_2]);
    normal_2 += (points[index_2 + width - 1] - points[index_2]).cross(points[index_2
      - 1] - points[index_2]);

    normal_1.normalize();
    normal_2.normalize();
    normal_points[index_1] = normal_1;
    normal_points[index_2] = normal_2;
  }

  /* Vypocet normal v rozich. */
  int index;
  SbVec3f normal;

  index = 0;
  normal = (points[index + 1] - points[index]).cross(points[index + width]
    - points[index]);
  normal.normalize();
  normal_points[index] = normal;

  index = (height * width) - 1;
  normal = (points[index - 1] - points[index]).cross(points[index - width]
    - points[index]);
  normal.normalize();
  normal_points[index] = normal;

  index = (height - 1) * width;
  normal = (points[index - width] - points[index]).cross(points[index - width + 1]
    - points[index]);
  normal += (points[index - width + 1] - points[index]).cross(points[index + 1]
    - points[index]);
  normal.normalize();
  normal_points[index] = normal;

  index = width - 1;
  normal += (points[index + width] - points[index]).cross(points[index + width - 1]
    - points[index]);
  normal += (points[index + width - 1] - points[index]).cross(points[index - 1]
    - points[index]);
  normal.normalize();
  normal_points[index] = normal;

  coords->point.finishEditing();
  texture_coords->point.finishEditing();
  normals->vector.finishEditing();
  simage_free_image(heightmap);

  /* Propojeni uzlu grafu sceny, */
  root->ref();
  root->addChild(style);
  root->addChild(separator);
  separator->addChild(terrain_callback);
  separator->addChild(style_callback);
  separator->addChild(camera);
  separator->addChild(light);
  separator->addChild(texture);
  separator->addChild(texture_coords);
  separator->addChild(coords);
  separator->addChild(normals);
  separator->addChild(normal_binding);

  switch (algorithm)
  {
    case ID_ALG_ROAM:
    {
      SoSimpleROAMTerrain * terrain = new SoSimpleROAMTerrain();
      terrain->mapSize.setValue(width);
      terrain->pixelError.setValue(pixel_error);
      terrain->triangleCount.setValue(triangle_count);
      terrain->frustrumCulling.setValue(is_frustrum_culling);
      terrain_callback->addEventCallback(SoKeyboardEvent::getClassTypeId(),
        terrainCallback, terrain);
      separator->addChild(terrain);
    }
    break;
    case ID_ALG_GEO_MIPMAP:
    {
      SoSimpleGeoMipmapTerrain * terrain = new SoSimpleGeoMipmapTerrain();
      terrain->mapSize.setValue(width);
      terrain->tileSize.setValue(tile_size);
      terrain->pixelError.setValue(pixel_error);
      terrain_callback->addEventCallback(SoKeyboardEvent::getClassTypeId(),
        terrainCallback, terrain);
      separator->addChild(terrain);
    }
    break;
    case ID_ALG_BRUAL_FORCE:
    {
      SoIndexedTriangleStripSet * terrain = new SoIndexedTriangleStripSet();

      /* Naplneni indexu terenu. */
      terrain->coordIndex.setNum((height - 1) * ((2 * width) + 1));
      int * indices = terrain->coordIndex.startEditing();

      int I = 0;
      for (int Y = 0; Y < (height - 1); Y++)
      {
        for (int X = 0; X < width; X++)
        {
          indices[I++] = (Y * width) + X;
          indices[I++] = ((Y + 1) * width) + X;
        }
        indices[I++] = -1;
      }
      terrain->coordIndex.finishEditing();
      separator->addChild(terrain);
    }
    break;
  }

  /* Nastaveni vlastnosti kamery a zobrazovaci plochy. */
  SoQtFreeViewer * render_area = new SoQtFreeViewer(window);
  SoSceneManager * scene_manager = new SoSceneManager();
  render_area->setHeadlight(FALSE);
  render_area->setSceneManager(scene_manager);
  scene_manager->setRenderCallback(renderCallback, render_area);
  scene_manager->activate();
  camera->viewAll(root, render_area->getViewportRegion());
  camera->nearDistance.setValue(0.01f);
  render_area->setSceneGraph(root);
  render_area->setTitle("SoTerrain Test Application");
  render_area->show();
  render_area->setFullScreen(is_full_screen);

  /* Spusteni animace. */
  SoTimerSensor * camera_timer = NULL;
  if (is_animation)
  {
    camera_timer = new SoTimerSensor(cameraTimerCallback,
      camera);
    camera_timer->setInterval(frame_time);
    camera_timer->schedule();
  }

  /* Spusteni aplikace */
  SoQt::show(window);
  SoQt::mainLoop();
  SoQt::done();

  PR_PRINT_RESULTS(profile_name);

  /* Uvolneni pameti. */
  root->unref();
  delete camera_timer;
  delete render_area;

  return 0;
}
