///////////////////////////////////////////////////////////////////////////////
//  SoTerrain
///////////////////////////////////////////////////////////////////////////////
/// Zkladn datov prvky algoritmu Geo Mip-mapping.
/// \file SbGeoMipmapPrimitives.cpp
/// \author Radek Barto - xbarto33
/// \date 30.01.2006
///
/// Zde jsou definovny zkladn datov typy algoritmu Geo Mip-Mapping.
/// Kvadrantov strom dladic ve td ::SbGeoMipmapTileTree rekurzivn
/// rozdluje vkovou mapu ternu na dladice tdy SbGeoMipmapTile. V jeho
/// hornch patrech jsou dladice pouze virtuln, tj. ty, kter neobsahuj
/// vlastn geometrii, ale jenom sv potomky. Dladice na nejni rovni
/// stromu obsahuj pole jednotlivch rovn detail dladice ve td
/// ::SbGeoMipmapTileLevel, jen u geometrii zahrnuj.
///////////////////////////////////////////////////////////////////////////////
//
// THIS SOFTWARE IS NOT COPYRIGHTED
//
// This source code is offered for use in the public domain.
//  You may use, modify or distribute it freely.
//
// This source code is distributed in the hope that it will be useful but
// WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
// DISCLAIMED.  This includes but is not limited to warranties of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// If you find the source code useful, authors will kindly welcome
// if you give them credit and keep their names with their source code.
//
///////////////////////////////////////////////////////////////////////////////

#include <geomipmapping/SbGeoMipmapPrimitives.h>

/******************************************************************************
* SbGeoMipmapTileLevel - public
******************************************************************************/

SbGeoMipmapTileLevel::SbGeoMipmapTileLevel(float _error, int * _vertices):
  error(_error), vertices(_vertices)
{
  // nic
}

SbGeoMipmapTileLevel::~SbGeoMipmapTileLevel()
{
  /* Uvolneni pameti. */
  delete[] vertices;
}

/******************************************************************************
* SbGeoMipmapTile - public
******************************************************************************/

/* Staticke konstanty. */
const int SbGeoMipmapTile::LEVEL_NONE = -1;

SbGeoMipmapTile::SbGeoMipmapTile(SbGeoMipmapTileLevel * _levels,
  SbGeoMipmapTile * _left, SbGeoMipmapTile * _right, SbGeoMipmapTile * _top,
  SbGeoMipmapTile * _bottom, int _level, SbBox3f _bounds, SbVec3f _center):
  levels(_levels), left(_left), right(_right), top(_top), bottom(_bottom),
  level(_level), bounds(_bounds), center(_center)
{
  // nic
}

SbGeoMipmapTile::~SbGeoMipmapTile()
{
  /* Uvolneni pameti. */
  delete[] levels;
}

/******************************************************************************
* SbGeoMipmapTileTree - public
******************************************************************************/

SbGeoMipmapTileTree::SbGeoMipmapTileTree(int _tile_count, int _tile_size):
  tile_count(_tile_count), tile_size(_tile_size), tree_size(0), level_count(0),
  level_sizes(NULL), tiles(NULL)
{
  /* Vypocet velikosti stromu dlazdic. */
  tree_size = tile_count;
  int tmp = tile_count;
  while (tmp > 1)
  {
    tmp >>= 2;
    tree_size+= tmp;
  }

  /* Vypocet zacatku dlazdic na nejnizzsi urovni. */
  bottom_start = tree_size - tile_count;

  /* Vypocet poctu urovni detailu v jedne dlazdici. */
  level_count = ilog2(tile_size - 1) + 1;

  /* Alokace a inicializace pole velikosti jednotlivych urovni detailu
  jedne dlazdice. */
  level_sizes = new int[level_count];
  level_sizes[0] = tile_size;
  for (int I = 1; I < level_count; ++I)
  {
    // predpoklad licheho cisla
    level_sizes[I] = (level_sizes[I - 1] >> 1) + 1;
  }

  /* Alokace pole dlazdic. */
  tiles = new SbGeoMipmapTile[tree_size];
}

SbGeoMipmapTileTree::~SbGeoMipmapTileTree()
{
  delete[] level_sizes;
  delete[] tiles;
}

/******************************************************************************
* SbGeoMipmapTileTree - private
******************************************************************************/

SbGeoMipmapTileTree::SbGeoMipmapTileTree(const SbGeoMipmapTileTree & old_tree)
{
  // nic
}
