///////////////////////////////////////////////////////////////////////////////
//  SoTerrain
///////////////////////////////////////////////////////////////////////////////
/// Tda roziujc tdu SoGroup o monosti profilovn.
/// \file SoProfileGroup.cpp
/// \author Radek Barto - xbarto33
/// \date 18.04.2006
///
/// Umonuje zmit dobu strvenou vykreslovnm uzl obsaench v tto skupin
/// za pomoc profileru ve statick tde ::PrProfiler.
///////////////////////////////////////////////////////////////////////////////
//
// THIS SOFTWARE IS NOT COPYRIGHTED
//
// This source code is offered for use in the public domain.
//  You may use, modify or distribute it freely.
//
// This source code is distributed in the hope that it will be useful but
// WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
// DISCLAIMED.  This includes but is not limited to warranties of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// If you find the source code useful, authors will kindly welcome
// if you give them credit and keep their names with their source code.
//
///////////////////////////////////////////////////////////////////////////////

#include <profiler/SoProfileGroup.h>

#ifdef PROFILE

SO_NODE_SOURCE(SoProfileGroup)

/******************************************************************************
* SoProfileGroup - public
******************************************************************************/

void SoProfileGroup::initClass()
{
  /* Inicializace tridy. */
  SO_NODE_INIT_CLASS(SoProfileGroup, SoGroup, "Group");
}


SoProfileGroup::SoProfileGroup():
  ticks_per_usec(0), last_render_time(0.0)
{
  /* Inicializace tridy. */
  SO_NODE_CONSTRUCTOR(SoProfileGroup);
}

SoProfileGroup::SoProfileGroup(int numChildren):
  SoGroup(numChildren),
  ticks_per_usec(0), last_render_time(0.0)
{
  /* Inicializace tridy. */
  SO_NODE_CONSTRUCTOR(SoProfileGroup);
}

void SoProfileGroup::GLRender(SoGLRenderAction * action)
{
  /* Zmereni doby trvani rendrovani. */
  PR_START_OBJ_PROFILE(SoProfileGroup_GLRender, this);
  SoGroup::GLRender(action);
  PR_STOP_OBJ_PROFILE(SoProfileGroup_GLRender, this);

  /* Vypocet teto doby v mirkosekundach. */
  long count = PrProfiler::getResultCount();
  PrTimeStamp time = PrProfiler::getResult(count - 1).time_stamp
    - PrProfiler::getResult(count - 2).time_stamp;
  last_render_time = SbTime(0, time);
}

SbTime SoProfileGroup::getLastRenderTime() const
{
  // vraceni casu posledniho vykreslovani
  return last_render_time;
}

/******************************************************************************
* SoProfileGroup - private
******************************************************************************/

SoProfileGroup::~SoProfileGroup()
{
  // nic
}

#else // PROFILE

SO_NODE_SOURCE(SoProfileGroup)

/******************************************************************************
* SoProfileGroup - public
******************************************************************************/

void SoProfileGroup::initClass()
{
  /* Inicializace tridy. */
  SO_NODE_INIT_CLASS(SoProfileGroup, SoGroup, "Group");
}


SoProfileGroup::SoProfileGroup()
{
  /* Inicializace tridy. */
  SO_NODE_CONSTRUCTOR(SoProfileGroup);
}

SoProfileGroup::SoProfileGroup(int numChildren):
  SoGroup(numChildren)
{
  /* Inicializace tridy. */
  SO_NODE_CONSTRUCTOR(SoProfileGroup);
}

/******************************************************************************
* SoProfileGroup - private
******************************************************************************/

SoProfileGroup::~SoProfileGroup()
{
  // nic
}

#endif // PROFILE
