///////////////////////////////////////////////////////////////////////////////
//  Profilovaci knihovna
///////////////////////////////////////////////////////////////////////////////
/// Trida rozsirujici tridu SoSceneManager o moznosti profilovani.
/// \file SoProfileSceneManager.cpp
/// \author Radek Barto - xbarto33
/// \date 05.07.2005
///
/// Tento manaer scny umouje zjistit jakou dobu strvil procesor potae
/// vykreslovnn scny. Pro pouit je teba nahradit vchoz manaer scny
/// tmto a inicializovat profilovac knihovnu ve td ::PrProfiler.
///////////////////////////////////////////////////////////////////////////////
//
// THIS SOFTWARE IS NOT COPYRIGHTED
//
// This source code is offered for use in the public domain.
//  You may use, modify or distribute it freely.
//
// This source code is distributed in the hope that it will be useful but
// WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
// DISCLAIMED.  This includes but is not limited to warranties of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// If you find the source code useful, authors will kindly welcome
// if you give them credit and keep their names with their source code.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef PROFILE

#include <profiler/SoProfileSceneManager.h>

/******************************************************************************
* PrProfiler
*******************************************************************************
* public
******************************************************************************/

SoProfileSceneManager::SoProfileSceneManager():
  SoSceneManager(), ticks_per_usec(0), last_render_time(0, 0)
{
  /* Zjisteni poctu taktu CPU za mikrosekundu */
  ticks_per_usec = PrProfiler::getTicksPerUsec();
}

SoProfileSceneManager::~SoProfileSceneManager()
{
  // nic
}

void SoProfileSceneManager::render(const SbBool clearwindow,
  const SbBool clearzbuffer)
{
  /* Zmereni doby trvani rendrovani. */
  PR_START_OBJ_PROFILE(SoProfileSceneManager_render, this);
  SoSceneManager::render(clearwindow, clearzbuffer);
  PR_STOP_OBJ_PROFILE(SoProfileSceneManager_render, this);

  /* Vypocet teto doby v mirkosekundach. */
  long count = PrProfiler::getResultCount();
  PrTimeStamp time = PrProfiler::getResult(count - 1).time_stamp
    - PrProfiler::getResult(count - 2).time_stamp;
  last_render_time = SbTime(0, time);
}

SbTime SoProfileSceneManager::getLastRenderTime() const
{
  return last_render_time; // vraceni doby posledniho vykresleni
}

#endif
