///////////////////////////////////////////////////////////////////////////////
//  SoTerrain
///////////////////////////////////////////////////////////////////////////////
/// Prioritn fronta diamant na spojen.
/// \file SbROAMMergeQueue.cpp
/// \author Radek Barto - xbarto33
/// \date 25.08.2005
///
/// Do tto prioritn fronty se vkldaj diamanty, kter lze spojit na dva
/// trojhelnky a tm dva trojhelnky z pvodnch ty z triangulace odstranit.
/// Toto se provd tehdy, pokud je teba triangulaci zjednoduit a to tak, e
/// se vyjme diamant s minimln prioritou, spoj se a vloi se zpt ppadn
/// vznikl nov diamanty.
///////////////////////////////////////////////////////////////////////////////
//
// THIS SOFTWARE IS NOT COPYRIGHTED
//
// This source code is offered for use in the public domain.
//  You may use, modify or distribute it freely.
//
// This source code is distributed in the hope that it will be useful but
// WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
// DISCLAIMED.  This includes but is not limited to warranties of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// If you find the source code useful, authors will kindly welcome
// if you give them credit and keep their names with their source code.
//
///////////////////////////////////////////////////////////////////////////////

#include <roam/SbROAMMergeQueue.h>
#include <roam/SbROAMPrimitives.h>

/******************************************************************************
* SbROAMMergeQueue - public
******************************************************************************/

SbROAMMergeQueue::SbROAMMergeQueue(const int init_size):
  SbHeap(heap_funcs, init_size)
{
  // nic
}

SbROAMMergeQueue::~SbROAMMergeQueue()
{
  // nic
}

void SbROAMMergeQueue::emptyQueue()
{
  SbHeap::emptyHeap(); // vyprazdneni fronty
}

void SbROAMMergeQueue::add(SbROAMMergeQueueDiamond * diamond)
{
  /* Vlozeni diamondu a aktualizace indexu ve fronte. */
  int index = SbHeap::add(diamond);
  diamond->index = index;
}

void SbROAMMergeQueue::remove(SbROAMMergeQueueDiamond * diamond)
{
  SbHeap::remove(diamond->index); // odstraneni diamondu z fronty
}

SbROAMMergeQueueDiamond * SbROAMMergeQueue::remove(
  SbROAMSplitQueueTriangle * triangle)
{
  /* Patri-li trojuhelnik pod nejaky diamond. */
  SbROAMMergeQueueDiamond * diamond = triangle->diamond;
  if (diamond != NULL)
  {
    /* Vsechny trojuhelniky diamondu uz nejsou jeho soucasti. */
    diamond->first->diamond = NULL;
    diamond->second->diamond = NULL;

    /* Nejde-li od degradovany diamond. */
    if (diamond->third != NULL)
    {
      diamond->third->diamond = NULL;
      diamond->fourth->diamond = NULL;
    }
    SbHeap::remove(diamond->index); // odstraneni diamondu z fronty
  }
  return diamond;
}

SbROAMMergeQueueDiamond * SbROAMMergeQueue::extractMin()
{
  /* Vyber trojuhelniku s maximalni prioritou. */
  return reinterpret_cast<SbROAMMergeQueueDiamond *>(SbHeap::extractMin());
}

SbROAMMergeQueueDiamond * SbROAMMergeQueue::getMin()
{
  /* Vraceni trojuhelniku s maximalni prioritou. */
  return reinterpret_cast<SbROAMMergeQueueDiamond *>(SbHeap::getMin());
}

SbROAMMergeQueueDiamond * SbROAMMergeQueue::operator[](const int index)
{
  return reinterpret_cast<SbROAMMergeQueueDiamond *>(SbHeap::operator[](index));
}

/******************************************************************************
* SbROAMMergeQueue - protected
******************************************************************************/

SbHeapFuncs SbROAMMergeQueue::heap_funcs = {getDiamondPriority, getDiamondIndex,
  setDiamondIndex};

float SbROAMMergeQueue::getDiamondPriority(void * diamond)
{
  /* Zjisteni priority diamondu ve fronte. */
  return reinterpret_cast<SbROAMMergeQueueDiamond * >(diamond)->priority;
}

int SbROAMMergeQueue::getDiamondIndex(void * diamond)
{
  /* Zjisteni priority diamondu ve fronte. */
  return reinterpret_cast<SbROAMMergeQueueDiamond * >(diamond)->index;
}

void SbROAMMergeQueue::setDiamondIndex(void * diamond, int index)
{
  /* Nataveni priority diamondu ve fronte. */
  reinterpret_cast<SbROAMMergeQueueDiamond * >(diamond)->index = index;
}
