///////////////////////////////////////////////////////////////////////////////
//  SoTerrain
///////////////////////////////////////////////////////////////////////////////
/// Prioritn fronta trojhelnk na rozdlen.
/// \file SbROAMSplitQueue.cpp
/// \author Radek Barto - xbarto33
/// \date 25.08.2005
///
/// Do tto prioritn fronty se vkldaj trojhelnky, kter je teba zaadit
/// do triangulace. Pokud je teba triangulaci dle zjemovat, vyjme se z fronty
/// trojhelnk s maximln prioritou a vlo se zpt jeho potomci.
///////////////////////////////////////////////////////////////////////////////
//
// THIS SOFTWARE IS NOT COPYRIGHTED
//
// This source code is offered for use in the public domain.
//  You may use, modify or distribute it freely.
//
// This source code is distributed in the hope that it will be useful but
// WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
// DISCLAIMED.  This includes but is not limited to warranties of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// If you find the source code useful, authors will kindly welcome
// if you give them credit and keep their names with their source code.
//
///////////////////////////////////////////////////////////////////////////////

#include <roam/SbROAMSplitQueue.h>
#include <roam/SbROAMPrimitives.h>

/******************************************************************************
* SbROAMSplitQueue - public
******************************************************************************/

SbROAMSplitQueue::SbROAMSplitQueue(const int init_size):
  SbHeap(heap_funcs, init_size)
{
  // nic
}

SbROAMSplitQueue::~SbROAMSplitQueue()
{
  // nic
}

void SbROAMSplitQueue::emptyQueue()
{
  SbHeap::emptyHeap(); // vyprazdneni fronty
}

void SbROAMSplitQueue::add(SbROAMSplitQueueTriangle * triangle)
{
  /* Vlozeni trojuhelniku a aktualizace indexu ve fronte. */
  int index = SbHeap::add(triangle);
  triangle->index = index;
}

void SbROAMSplitQueue::remove(SbROAMSplitQueueTriangle * triangle)
{
  SbHeap::remove(triangle->index); // odstraneni trojuhelniku z fronty
}

SbROAMSplitQueueTriangle * SbROAMSplitQueue::extractMax()
{
  /* Vyber trojuhelniku s maximalni prioritou. */
  return reinterpret_cast<SbROAMSplitQueueTriangle *>(SbHeap::extractMin());
}

SbROAMSplitQueueTriangle * SbROAMSplitQueue::getMax()
{
  /* vraceni trojuhelniku s maximalni prioritou. */
  return reinterpret_cast<SbROAMSplitQueueTriangle *>(SbHeap::getMin());
}

SbROAMSplitQueueTriangle * SbROAMSplitQueue::operator[](const int index)
{
  return reinterpret_cast<SbROAMSplitQueueTriangle*>
    (SbHeap::operator[](index));
}

/******************************************************************************
* SbROAMSplitQueue - private
******************************************************************************/

SbHeapFuncs SbROAMSplitQueue::heap_funcs = {getTrianglePriority,
  getTriangleIndex, setTriangleIndex};

inline float SbROAMSplitQueue::getTrianglePriority(void * triangle)
{
  /* Zjisteni priority trojuhelniku ve fronte. */
  return reinterpret_cast<SbROAMSplitQueueTriangle *>(triangle)->priority;
}

inline int SbROAMSplitQueue::getTriangleIndex(void * triangle)
{
  /* Zjisteni priority trojuhelniku ve fronte. */
  return reinterpret_cast<SbROAMSplitQueueTriangle *>(triangle)->index;
}

inline void SbROAMSplitQueue::setTriangleIndex(void * triangle, int index)
{
  /* Nataveni priority trojuhelniku ve fronte. */
  reinterpret_cast<SbROAMSplitQueueTriangle *>(triangle)->index = index;
}
